"""barrel.cooper - convenience interface for barrel

(See the docstrings of the various functions and classes.)

Copyright (C) 2006 Luke Arno - http://lukearno.com/

This program is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your 
option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

The Free Software Foundation, Inc., 
51 Franklin Street, Fifth Floor, 
Boston, MA  02110-1301, USA.

Luke Arno can be found at http://lukearno.com/

"""


from barrel.basic import BasicAuth
from barrel.form import FormAuth
from barrel.combo import BasicFormAuth
from barrel.roles import RolesAuthz


def decorize(middleware):
    """Return customizable middleware decorator factory.
    
    Returns a decorator factory function that will return
    decorators that apply the given middleware then assign
    any keyword args to corresponding attributes of the result.
    """
    def metadeco(**options):
        """Decorate with middleware then set attribs."""
        def deco(app):
            """Decorated with middleware and attribs set."""
            newapp = middleware(app)
            for key, value in options:
                try:
                    setattr(newapp, key, value)
                except AttributeError:
                    pass # Attribute error passes silently.
            return newapp
        return deco
    return metadeco


basicauth = decorize(BasicAuth)
formauth = decorize(FormAuth)
comboauth = decorize(BasicFormAuth)
rolesauth = decorize(RolesAuthz)

